=== added directory 'doc'
=== added directory 'doc/draft'
=== added file 'doc/draft/charms-in-action.txt'
--- doc/draft/charms-in-action.txt	1970-01-01 00:00:00 +0000
+++ doc/draft/charms-in-action.txt	2012-12-12 10:14:17 +0000
@@ -0,0 +1,320 @@
+Charms in action
+================
+
+This document describes the behaviour of the go implementation of the unit
+agent, whose behaviour differs in some respects from that of the python
+implementation. This information is largely relevant to charm authors, and
+potentially to developers interested in the unit agent.
+
+Hooks
+-----
+
+A charm's direct action is entirely defined by its hooks. Hooks are executable
+files in a charm's hooks directory; hooks with particular names will be invoked
+by juju at particular times, and thereby cause the charm to run a unit of its
+service and respond to changes in its environment.
+
+Whenever a hook-worthy action takes place, juju tries to run the hook with the
+appropriate name. If the hook doesn't exist, juju continues without error; if
+it exists, it is invoked without arguments in a specific environment, and its
+output is written to the unit's log. If it returns a non-zero exit code, juju
+puts the unit into an error state and awaits resolution; otherwise it continues
+to process environment changes as before.
+
+In general, a unit will run hooks in a clear sequence, about which a number of
+useful guarantees can and will be made. All such guarantees come with the caveat
+that there is [TODO: will be: `remove-unit --force`] a mechanism for forcible
+termination of a unit, and that a unit so terminated will just stop, dead, and
+completely fail to run anything else ever again. This shouldn't actually be a
+big deal in practice.
+
+Errors in hooks
+---------------
+
+Hooks should be idempotent, because they can fail, and may need to be re-executed
+from scratch. As a hook author, you don't have complete control over the times
+your hook might be stopped: if the unit agent process is killed for any reason
+while running a hook, then when it recovers it will treat that hook as having
+failed, just as if it had returned a non-zero exit code, and request user
+intervention.
+
+It is unrealistic to expect great sophistication on the part of the average user,
+and as a charm author you should expect that users will attempt to re-execute
+failed hooks before attempting to investigate or understand the situation. You
+should therefore make every effort to ensure your hooks are idempotent when
+aborted and restarted.
+
+[TODO: I have a vague feeling that `juju resolved` actually defaults to "just
+pretend the hook ran successfully" mode. I'm not sure that's really the best
+default, but I'm also not sure we're in a position to change the UI that much.]
+
+The most sophisticated charms will consider the nature of its operations with
+care, and will be prepared to internally retry any operations it suspects of
+having failed transiently, to ensure that it only requests user intervention in
+the most trying circumstances; and will also be careful to log any relevant
+information or advice before signalling the error.
+
+[TODO: I just thought; it would be really nice to have a juju-fail hook tool,
+which would allow charm authors to explicity set the unit's error status to
+something a bit more sophisticated than "X hook failed". Wishlist, really.]
+
+Charm deployment
+----------------
+
+  * A charm is deployed into a directory, managed by git, that is entirely
+    owned and controlled by juju.
+  * At certain times, control of the directory is ceded to the charm (by
+    running a hook) or to the user (by entering an error state).
+  * At these times, and only at these times, should the charm directory be
+    used by anything other than juju itself.
+
+The most important consequence of this is that it is a mistake to conflate the
+state of the charm with the state of the software deployed by the charm: it's
+fine to store *charm* state in the charm directory, but the charm must deploy
+its actual software elsewhere on the system.
+
+To put it another way: once a charm has put the system into a state, the system
+should remain in that state completely independently of the charm's existence,
+and there is no mechanism by which deployed software can safely feed information
+back into the charm of its own accord.
+
+[TODO: this sucks a bit. We have plans for a tool called `juju-run`, which
+would allow an arbitrary script to be invoked as though it were a hook at any
+time (well, it'd block until no other hook were running, but still). Probably
+isn't even that hard but it's still rolling around my brain, might either click
+soon or be overridden by higher priorities and be left for ages. I'm less sure,
+but have a suspicion, that `juju ssh <unit>` should also default to a juju-run
+environment: primarily because, without this, in the context of forced upgrades,
+the system cannot offer *any* guarantees about what it might suddenly do to the
+charm directory while the user's doing things with it. The alternative is to
+allow unguarded ssh, but tell people that they have to use something like
+`juju-run --interactive` before they modify the charm dir; this feels somewhat
+user-hostile, though.]
+
+Execution environment
+---------------------
+
+Every hook is run in the deployed charm directory, in an environment with the
+following characteristics:
+
+  * $PATH is prefixed by a directory containing command line tools through
+    which the hooks can interact with juju.
+  * $CHARM_DIR holds the path to the charm directory.
+  * $JUJU_UNIT_NAME holds the name of the local unit.
+  * $JUJU_CONTEXT_ID and $JUJU_AGENT_SOCKET are set (but should not be messed
+    with: the command line tools won't work without them).
+
+Hook tools
+----------
+
+All hooks can directly use the following tools:
+
+  * juju-log (write arguments direct to juju's log (potentially redundant, hook
+    output is all logged anyway, but --debug may remain useful))
+  * unit-get (returns the local unit's private-address or public-address)
+  * open-port (marks the supplied port/protocol as ready to open when the
+    service is exposed)
+  * close-port (reverses the effect of open-port)
+  * config-get (get current service configuration values)
+  * relation-get (get the settings of some related unit)
+  * relation-set (write the local unit's relation settings)
+  * relation-ids (list all relations using a given charm relation)
+  * relation-list (list all units of a related service)
+
+Within the context of a single hook execution, the above tools present a
+sandboxed view of the system with the following properties:
+
+  * Any data retrieved corresponds to the real value of the underlying state at
+    some point in time.
+  * Data is only written back to state when the hook completes without error;
+    changes made by a failing hook will be discarded.
+  * Not actually sandboxed: open-port and close-port operate directly on state.
+    [TODO: this is a stupid bug that I must capture properly.]
+
+Hook kinds
+----------
+
+There are 5 `unit hooks` with predefined names that can be implemented by any
+charm:
+
+  * install
+  * start
+  * config-changed
+  * upgrade-charm
+  * stop
+
+For every relation defined by a charm, an additional 4 `relation hooks` can be
+implemented, named after the charm relation:
+
+  * <name>-relation-joined
+  * <name>-relation-changed
+  * <name>-relation-departed
+  * <name>-relation-broken
+
+Unit hooks
+----------
+
+The `install` hook always runs once, and only once, before any other hook.
+
+The `start` hook always runs once immediately after the install hook; there are
+currently no other circumstances in which it will be called, but this may change
+in the future.
+
+The `config-changed` hook always runs once immediately after the start hook,
+and likewise after the upgrade-charm hook. It also runs whenever the service
+configuration changes, and when recovering from transient errors.
+
+The `upgrade-charm` hook always runs once immediately after the charm directory
+contents have been changed by an unforced charm upgrade operation, and *may* do
+so after a forced upgrade; but will *not* be run after a forced upgrade from an
+existing error state.
+
+The `stop` hook is the last hook to be run before the unit is destroyed. In the
+future, it may be called in other situations.
+
+In normal operation, a unit will run at least the install, start, config-changed
+and stop hooks over the course of its lifetime.
+
+It should be noted that, while all hook tools are available to all hooks, the
+relation-* tools are not useful to the install, start, and stop hooks; this is
+because the first two are run before the unit has any opportunity to participate
+in any relations, and the stop hooks will not be run while the unit is still
+participating in one.
+
+Relation hooks
+--------------
+
+For each charm relation, any or all of the 4 relation hooks can be implemented.
+Relation hooks operate in an environment slightly different to that of unit
+hooks, in the following ways:
+
+  * JUJU_RELATION is set to the name of the charm relation. This is of limited
+    value, because every relation hook already "knows" what charm relation it
+    was written for; that is, in the "foo-relation-joined" hook, JUJU_RELATION
+    is "foo".
+  * JUJU_RELATION_ID is more useful, because it serves as unique identifier for
+    a particular relation, and thereby allows the charm to handle distinct
+    relations over a single endpoint. In hooks for the "foo" charm relation,
+    JUJU_RELATION_ID always has the form "foo:<id>", where id uniquely but
+    opaquely identifies the runtime relation currently in play.
+  * The relation-* hook tools, which ordinarily require that a relation be
+    specified, assume they're being called with respect to the current
+    relation. The default can of course be overridden as usual.
+
+Furthermore, all relation hooks except relation-broken are notifications about
+some specific unit of a related service, and operate in an environment with the
+following additional properties:
+
+  * JUJU_REMOTE_UNIT is set to the name of the current related unit.
+  * The relation-get hook tool, which ordinarily requires that a related unit
+    be specified, assumes that it is being called with respect to the current
+    related unit. The default can of course be overridden as usual.
+
+For every relation in which a unit partcipates, hooks for the appropriate charm
+relation are run according to the following rules.
+
+The "relation-joined" hook always runs once when a related unit is first seen.
+
+The "relation-changed" hook for a given unit always runs once immediately
+following the relation-joined hook for that unit, and subsequently whenever
+the related unit changes its settings (by calling relation-set and exiting
+without error). Note that "immediately" only applies within the context of
+this particular runtime relation -- that is, when "foo-relation-joined" is
+run for unit "bar/99" in relation id "foo:123", the only guarantee is that
+the next hook to be run *in relation id "foo:123"* will be "foo-relation-changed"
+for "bar/99". Unit hooks may intervene, as may hooks for other relations,
+and even for other "foo" relations.
+
+The "relation-departed" hook for a given unit always runs once when a related
+unit is no longer related. After the "relation-departed" hook has run, no
+further notifications will be received from that unit; however, its settings
+will remain accessible via relation-get for the complete lifetime of the
+relation.
+
+The "relation-broken" hook is not specific to any unit, and always runs once
+when the local unit is ready to depart the relation itself. Before this hook
+is run, a relation-departed hook will be executed for every unit known to be
+related; it will never run while the relation appears to have members, but it
+may be the first and only hook to run for a given relation. The stop hook will
+not run while relations remain to be broken.
+
+Relations in depth
+------------------
+
+A unit's `scope` consists of the group of units that are transitively connected
+to that unit within a particular relation. So, for a globally-scoped relation,
+that means every unit of each service in the relation; for a locally-scoped
+relation, it means only those sets of units which are deployed alongside one
+another.  That is to say: a globally-scoped relation has a single unit scope,
+whilst a locally-scoped relation has one for each principal unit.
+
+When a unit becomes aware that it is a member of a relation, its only self-
+directed action is to `join` its scope within that relation. This involves two
+steps:
+
+  * Write initial relation settings (just one value, "private-address"), to
+    ensure that they will be available to observers before they're triggered
+    by the next step;
+  * Signal its existence, and role in the relation, to the rest of the system.
+
+The unit then starts observing and reacting to any other units in its scope
+which are playing a role in which it is interested. (Providers and requirers
+only observe each other; peers observe all peers which are not themselves.)
+
+Now, suppose that unit was the very first unit to join the relation; and let's
+say it's a requirer. No provider units are present, so no hooks will fire. But,
+when a provider unit joins the relation, the requirer and provider become aware
+of each other almost simultaneously. (Similarly, the first two units in a peer
+relation become aware of each other almost simultaneously.)
+
+So, concurrently, the units on each side of the relation run their relation-joined
+and relation-changed hooks with respect to their counterpart. The intent is that
+they communicate appropriate information to each other to set up some sort of
+connection, by using the relation-set and relation-get hook tools; but neither
+unit is safe to assume that any particular setting has yet been set by its
+counterpart.
+
+This sounds kinda tricky to deal with, but merely requires disciplined use of
+the relation-get tool. Advanced users may on occasion have cause to break the
+following rules, but such situations are outside the scope of this document.
+
+  * In relation-changed hooks, use relation-get freely on units of the current
+    relation (those returned by calling relation-list without arguments), but
+    be prepared for missing data. This is best handled by exiting silently,
+    without error: there is no cause for alarm, because the hook will be run
+    again as soon as the related unit writes its settings.
+  * In relation-joined hooks, use only `relation-get private-address`.
+  * Otherwise, don't use relation-get.
+
+In general, where it is practical to do so, it is also a good idea to restrict
+use of relation-set to the relation-joined hook. This is by no means required,
+or even guaranteed to be possible, but it is the usage pattern that integrates
+most smoothly with the system, and is most likely to avoid causing fruitlessly
+repeated executions of relation-changed hooks by counterpart units.
+
+Departing relations
+-------------------
+
+A unit will depart a relation when either the relation or the unit itself is
+marked for termination. In either case, it follows the same sequence:
+
+  * For every known related unit -- those which have joined and not yet
+    departed -- run the relation-departed hook.
+  * Run the relation-broken hook.
+  * `depart` from its scope in the relation.
+
+The unit's departure from its scope will in turn be detected by units of the
+related service, and cause them to run relation-departed hooks. A unit's
+relation settings persist beyond its own departure from the relation; the
+final unit to depart a relation marked for termination is responsible for
+destroying the relation and all associated data.
+
+Debugging charms
+----------------
+
+Facilities are currently not good.
+
+  * juju ssh
+  * juju debug-hooks [TODO: not implemented]
+  * juju debug-log [TODO: not implemented]
+

=== added file 'doc/draft/glossary.txt'
--- doc/draft/glossary.txt	1970-01-01 00:00:00 +0000
+++ doc/draft/glossary.txt	2012-12-12 10:14:17 +0000
@@ -0,0 +1,357 @@
+Juju Glossary
+=============
+
+This document introduces and briefly explains a number of terms that are used
+throughout the juju documentation, roughly broken down by conceptual area. The
+intent of this document is to make the reader aware of the available modes of
+interaction with juju, and how the various commands and concepts interact. It
+makes no attempt to be comprehensive; just accurate, and opinionated.
+
+Charms
+------
+
+A `charm` is a collection of code and data that embodies best practice in
+the deployment of a particular piece (or collection) of software. Examples
+of software that has been charmed include:
+
+  * wordpress (blogging platform)
+  * mediawiki (wiki platform)
+  * mysql (relational database)
+  * mongodb (non-relational database)
+  * hadoop (heavyweight data crunching)
+  * glance, keystone, nova-compute, etc (openstack components)
+  * minecraft (game server)
+
+A charm consists of the following compnents:
+
+  * metadata (describes the charm's purpose and capabilities)
+  * configuration (describes the ways in which a user can tune the software)
+  * hooks (executables, invoked by juju, that configure and deploy the software)
+  * revision (an integer identifying separate versions of the same charm)
+  * any additional code or data useful to the hooks or the deployed software
+
+A `charm directory` is a filesystem directory containing the aforementioned
+components of a charm in standard locations. (Any additional code/data can go
+anywhere not reserved for the other components.)
+
+A `charm bundle` is a charm directory serialized as a zip file. This format is
+used for storage and distribution only; when a charm is deployed it is always
+unbundled into a charm directory, within which all hooks are executed.
+
+A `repository` is a collection of charms that can be deployed by juju.
+
+The `charm store` is the default repository, which serves only curated bundled
+charms.
+
+A `local repository` is a repository located on the same system as the juju
+client.
+
+A `charm URL` is a string that identifies the provenance and the intended
+deployment target of a charm, and may also specify the charm's revision.
+Charms are identified by their charm URLs.
+
+Environments
+------------
+
+A `machine` is a compute resource on which components of juju, and of the
+software deployed by juju, can run. The following command is used to
+manipulate machines directly.
+
+  * juju terminate-machine (soon to alias destroy-machine) [TODO: not implemented]
+
+An `environment` is a deployment of juju. It always includes at least one
+machine responsible for maintaining the system's state, and potentially
+provisioning additional machines in response to state changes. The following
+commands are used to manipulate and inspect the environment.
+
+  * juju bootstrap
+  * juju status [TODO: somewhat incomplete]
+  * juju upgrade-juju
+  * juju destroy-environment
+
+An `environment provider` mediates between juju and the substrate on which an
+environment runs. Each provider allows juju to run against a different backend:
+
+  * ec2 (Amazon EC2 and S3)
+  * maas (bare metal) [TODO: not implemented]
+  * local (lightweight containers on client machine) [TODO: not implemented]
+  * openstack (many public and private clouds) [TODO: not implemented]
+
+An `environment configuration` describes how to create and connect to an
+environment from a specific provider. An environment configuration must
+always specify at least the following information:
+
+  * name (to identify the environment)
+  * type (to specify the provider)
+  * admin-secret (a "password" identifying an client with administrative-
+    level access to system state)
+  * authorized-keys (SSH public keys identifying users allowed to connect to
+    machines in the environment)
+
+...and may accept or even require additional keys depending on the provider
+type; but the full details of environment configuration are outside the scope
+of this document.
+
+[TODO: there should/will be commands for inspecting and manipulating the
+configuration of an environment while it's running; the names "env-get" and
+"env-set" have been mooted, but I'm not sure that's sane: I can't see any
+reason not to use the existing "get" and "set", which currently only work
+for services. But I could just be missing something obvious. Note that certain
+settings, such as "name" and "type", and "region" in the ec2 provider, will
+need to be immutable, and I'm not sure whether that's implemented yet.]
+
+An `environments file` is a YAML file containing environment configurations.
+Juju cannot operate without a valid environments file, which must be saved at
+`~/.juju/environments.yaml` on the client machine.
+
+The simplest functional environments file looks like this:
+
+  environments:
+    aws:
+      type: ec2
+      admin-secret: <password string>
+      control-bucket: <globally unique S3 bucket name>
+
+The "name" of the single defined environment is determined by its key in the
+environments map ("aws"), and the "authorized-keys" can be safely omitted so
+long as the client machine has a public key with a standard name saved in
+`~/.ssh`. Because it's using the "ec2" provider type, it also has to specify
+a "control-bucket" key; the "access-key" and "secret-key" settings are also
+required, but can be omitted if AWS_ACCESS_KEY_ID and AWS_SECRET_ACCESS_KEY
+are set in the client process environment. The "region" setting defaults to
+"us-east-1"; there are other possible settings but they're not relevant here.
+
+[NOTE: there is much more to environment configuration than has been discussed
+here. "authorized-keys-path" and "default-series" both apply to every provider,
+the ec2 provider offers many not mentioned here, and it will only get worse with
+each new provider...]
+
+Services
+--------
+
+A `service` is a deployment of a charm in an environment. The following commands
+are used to manipulate services:
+
+  * juju deploy
+  * juju destroy-service [TODO: not implemented]
+
+A service's `configuration` is defined by its charm, and allows the user to
+inspect and tune the service's operation using the following commands:
+
+  * juju get
+  * juju set
+
+A `unit` is the fundamental component of a service; services are composed of
+units. Each unit represents an instance of the charm's software, deployed to
+some machine in the service's environment. The following commands are used to
+manipulate units:
+
+  * juju add-unit
+  * juju remove-unit (soon to alias destroy-unit)
+
+[TODO: units are currently not, but will shortly be, deployed to LXC containers
+within their machines; this prevents units on the same machine from interfering
+with one another (except when each attempts to open the same port; we haven't
+worked out how to deal with that yet).]
+
+A service is `exposed` if it is theoretically accessible from the public
+internet (or, at least, from outside its environment). The fact of a service's
+exposure does not necessarily imply that any units of that service are actually
+accessible; actual access is mediated by the service's units, which are
+responsible for specifying the ports that should actually be opened when the
+service is exposed. Service exposure is controlled with the following commands:
+
+  * juju expose
+  * juju unexpose
+
+[NOTE: the "firewall-mode" environment configuration setting comes into play
+here, but that may be a topic for a more detailed document.]
+
+A service's charm can be `upgraded` when the revision of that charm available
+from its original repository is greater than that or the service's current charm,
+and when the following conditions hold:
+
+  * all configuration keys that exist in both versions of the charm must have
+    the same data type (addition or removal of config settings is fine, and it's
+    ok for defaults to change, but (eg) a string cannot become an integer).
+  * any normal relations in which the service is participating must also be
+    available, unchanged, in the newer version of the charm (addition or removal
+    of charm relations is, in general, fine).
+  * [TODO: something to do with storage compatibility, but storage doesn't exist
+    yet]
+
+The following command is used to upgrade a service's charm:
+
+  * juju upgrade-charm [TODO: not implemented]
+
+[TODO: --revision and --switch params, that do not exist in the python version,
+are mooted; they would respectively allow up/downgrades to specific revisions,
+and crossgrades to entirely different charms, but priority/agreement is unclear.]
+
+Sometimes (although, hopefully, rarely) a unit will encounter an `error` that
+requires human intervention. These cases are as follows:
+
+  * a charm hook returned a non-zero exit code
+  * the unit process was killed while running a charm hook
+  * a charm upgrade failed due to conflicting content in the charm directory
+
+In each of these situations, the unit stops responding to most external events,
+and waits for an administrator to resolve the problem. The administrator may
+need to log into the machine with the failed unit to determine and resolve the
+problem, or he may be able to resolve the problem automatically by trying to
+re-run the failed hook. The following commands are useful in error recovery:
+
+  * juju ssh
+  * juju scp
+  * juju resolved
+
+A `forced upgrade` is a form of upgrade that ignores unit error states, and
+upgrades them anyway. Forced upgrades are subtle and quick to anger; they
+are only recommended if you have sole control of your environment and a clear
+understanding of the upgrade process as it applies to your specific charm.
+
+Relations
+---------
+
+An `interface` is an informally-agreed protocol for transferring information
+between service units. Examples include:
+
+  * http (hostname, port)
+  * mysql (host, database, user, password, slave)
+  * ceph-client (key, auth, rid) [TODO: I don't think rid is appropriate, we
+    should take a look and figure out if it could be dropped]
+
+A `role` describes the manner in which a charm uses an interface. A role can
+have one of three values: "provider", "requirer", or "peer".
+
+A `(charm) relation` is an entry in a charm's metadata that indicates that it
+can fulfil some role over some interface.
+
+An `endpoint` is the combination of a service with one of its charm's relations.
+The set of a service's endpoints define the possible connections involving that
+service. Within an environment, an endpoint is uniquely identifiable when
+expressed in the form `<service-name>:<charm-relation-name>`.
+
+A "normal" `relation` is a connection between the endpoints of two services.
+The services' charms must respectively "provide" and "require" relations with
+identical interfaces. When two services are participating in a relation, each
+unit of each service can communicate with every unit of the other service.
+Normal relations can be manipulated with the following commands:
+
+  * juju add-relation
+  * juju remove-relation (soon to alias destroy-relation)
+
+A `(peer) relation` is a connection within a service defined by a single
+endpoint with the "peer" role. If such an endpoint exists, the peer relation
+is automatically created when the service is deployed; each unit of the
+service can communicate with every other unit of the service. Peer relations
+cannot be manipulated in the UI, but are displayed in `juju status` output.
+
+The word `relation`, when used casually, *may* refer to a charm relation or a
+peer relation, but is most likely to refer to a "normal" relation.
+
+When manipulating relations, it is important to understand that a single
+endpoint can participate in multiple relations. For example, a mysql:server
+endpoint can quite happily participate in relations with both a wikimedia:db
+endpoint and a wordpress:db endpoint (or even in multiple relations with
+separate services running their own charms: for example, wordpress1:db and
+wordpress2:db). In each case, the mysql charm is responsible for creating
+separate users and databases for each separate relation.
+
+[TODO: charm relations also have a concept of "limit" which probably ought to
+come in here, but no code respects it at the moment. Oh, and a "required" field
+too, and I'm not sure whether anyone fully recalls its precise intended
+semantics... not sure what to do about this.]
+
+Subordinates
+------------
+
+[TODO: subordinates are not yet implemented in go, but the vast majority of the
+building blocks are already integrated.]
+
+A `subordinate charm` is a charm which declares itself to be subordinate.
+
+A `subordinate service` is a service running a subordinate charm.
+
+A `subordinate unit` is a unit of a subordinate service.
+
+All other charms, services, and units are `principal` units, by virtue of not
+being subordinate.
+
+A charm relation's `scope` controls which units within a relation are visible
+to one another. Most relations have "global" scope, which is the default; a
+subordinate charm must define at least one relation with "local" scope.
+
+A normal relation's `scope` is the narrowest scope amongst its endpoints; a
+normal relation's scope is "global" unless either endpoint has "local" scope,
+in which case it the whole relation has "local" scope.
+
+When a subordinate service is deployed, no units are created; the add-unit and
+remove-unit commands do not apply to subordinate services. Instead, subordinate
+units are deployed as a side-effect of the creation of locally-scoped relations
+between the subordinate service and others.
+
+To clarify: when a new relation is added and all the following conditions apply:
+
+  * one service is a subordinate;
+  * one service is a principal;
+  * the relation between the two is locally scoped, by virtue of at least one
+    of the endpoints having local scope:
+
+...a new unit of the subordinate service will be created and deployed alongside
+each unit of the principal service. Each principal unit is responsible for its
+own associated subordinate, which runs alongside the principal with essentially
+identical privileges. Due to the local scoping of the relation, each subordinate
+unit responds only to the principal unit that deployed it, and vice versa.
+
+[TODO: to clarify: once units are deployed inside their own containers, subordinate
+units will be installed inside their principal unit's container. But we don't have
+containers yet.]
+
+A subordinate service can of course participate in globally-scoped relations as
+well; such relations can be added as normal, and the subordinate units participate
+in those relations just as in any other global relation.
+
+[NOTE: there's also a magic implicit charm relation, called "juju-info", which
+provides the "juju-info" interface and allows subordinates to establish relations
+with arbitrary principals. Couldn't really figure out how to work it in nicely.]
+
+Constraints
+-----------
+
+[TODO: constraints are not yet implemented in go; a substantial amount of work
+needs to be done]
+
+Some environment providers offer the ability to provision machines with varying
+characteristics. An environment provider defines a vocabulary of `constraints`
+to control the compute resources that are available from the provider; by
+specifying global environment constraints and overriding them where necessary
+at the service level, a user can ensure that her workloads are only run on
+machines with appropriate characteristics.
+
+Available constraints vary by provider, and are manipulated and inspected using
+the following commands:
+
+  * juju set-constraints (not implemented)
+  * juju get-constraints (not implemented)
+
+Storage
+-------
+
+[TODO: storage remains an informally-specified concept distributed across
+several brains; don't even try to document it.]
+
+Entity lifecycles
+-----------------
+
+When services, relations, units and machines are added and destroyed by the
+client, changes to the environment will take some time to occur: in particular,
+destruction is rarely instantaneous. Destroyed entities will continue to be
+displayed in juju status output but marked as "dying" [TODO: they are not] until
+the underlying resources have been removed.
+
+This is not really worth worrying about in general, but it does mean that after
+a service or relation has been destroyed you will need to wait until they have
+been *removed* before you're able to add a new service or relation with the same
+name. It's not an issue for units or machines, because their names are assigned
+internally by juju and are guaranteed to be unique.

=== added file 'doc/draft/lifecycles.txt'
--- doc/draft/lifecycles.txt	1970-01-01 00:00:00 +0000
+++ doc/draft/lifecycles.txt	2012-12-12 10:14:17 +0000
@@ -0,0 +1,281 @@
+Lifecycles
+==========
+
+In juju, some things have "lifecycles". We sometimes refer to these things as
+"entities", but I think that term is becoming overloaded to mean "thing that
+can connect to state", ie unit and machine agents (and then presumably, one day,
+users). So we don't have a good name for the group, but happily there are ony 4:
+
+  * Machines
+  * Units
+  * Services
+  * Relations
+
+...and there are only 4 possible states for the above things:
+
+  * Alive
+  * Dying
+  * Dead
+  * (removed)
+
+There are two fundamental truths in this system:
+
+  * All such things start existence Alive.
+  * No such thing can ever change to an earlier state.
+
+...and just about all else is tricky detail, that is worth examining from a few
+different angles. First of all, it's worth considering the things that happen to
+each state object separately, insofar as that is possible:
+
+Machines
+--------
+
+  * Like everything else, a machine starts out Alive.
+  * While it is Alive, principal units can be assigned to it.
+  * When no principal units are assigned to it, it can be destroyed with
+    `juju destroy-machine`. This will set the machine to Dying, but will
+    fail when units are assigned. (Future plans: allow a machine to become
+    Dying when it has principal units, so long as they are not Alive. For now
+    it's extra complexity with little direct benefit.)
+  * Once a machine has been set to Dying, the corresponding Machine Agent (MA)
+    is responsible for setting it to Dead. (Future plans: when Dying units are
+    assigned, wait for them to become Dead and remove them completely before
+    making the machine Dead; not an issue now because the machine can't yet
+    become Dying with units assigned.)
+  * When the Provisioning Agent (PA) observes that the machine has become Dead,
+    it releases the underlying instance back to the provider and removes the
+    machine object from state. (Future uncertainty: should the PA provision an
+    instance for a Dying machine? At the moment, no, because a Dying machine
+    can't have any units in the first place; in the future, er, maybe, because
+    those Dying units may be attached to persistent storage and should thus be
+    allowed to continue to shut down cleanly as they would usually do. Maybe.)
+
+Units
+-----
+
+  * While a principal unit is alive, it can be assigned to machines and can have
+    subordinate units added. When subordinate units are added, it is responsible
+    for them in exactly the same
+  * A unit can become Dying at any time, but may not become Dead while any unit
+    subordinate to it exists, or when the unit is in scope for any relation.
+    (Future plans: `juju destroy-unit --force`, which will clean up a unit and
+    all its subordinates and relations.)
+  * A unit can become Dying in one of two ways:
+      * `juju destroy-unit`
+      * `juju destroy-service` (This happens indirectly: the Unit Agents (UAs)
+        for each unit of a service set their corresponding units to Dying when
+        they detect service Dying; this is because we try to assume 100k-scale
+        and we can't use mgo/txn to do a bulk update of 100k units because that
+        makes for a transaction with at least 100k operations and that's just
+        crazy.)
+  * When a principal unit becomes Dying, all its subordinates must also become
+    Dying.
+  * When a unit is Dying, its UA is responsible for removing impediments to the
+    unit becoming Dead, and then making it so. To do so, the UA must:
+      * Depart from all its relations in an orderly fashion.
+      * Wait for all its subordinates to become Dead, and remove them from state.
+      * Set its unit to Dead.
+  * As just noted, when a subordinate unit is Dead, it is removed from state by
+    its principal's UA; similarly, when a principal unit is Dead, it is removed
+    from state by the MA of the machine to which it is assigned.
+
+Services
+--------
+
+  * Unlike units and machines, services have no corresponding agent.
+  * In addition, services are never Dead: they can only be Alive or Dying (or
+    removed, if you want to consider that a life state).
+  * When a service is Alive, units may be added to it, and the service's
+    endpoints can be added torelations.
+  * A service can become dying at any time, via `juju destroy-service`. This
+    causes all the units to become Dying, as discussed above, and will also
+    cause all relations in which the service is participating to become Dying.
+    (Contentious? I *think* python blocks service removal until relations are
+    removed...)
+  * If no associated units or relations exist when a service is destroyed, the
+    service is removed from state immediately rather than being set to Dying.
+  * If no associated relations exist, the service is removed by the MA which
+    removes the last unit of that service from state.
+  * If no units of the service remain, but its relations still exist, the
+    responsibility for removing the service falls to the last UA to leave scope
+    for that relation. (Yes, this is a UA for a unit of a totally different
+    service.)
+
+Relations
+---------
+
+  * A relation, like a service, has no corresponding agent and cannot be Dead.
+  * While a relation is Alive, units of services in that relation can enter its
+    scope.
+  * A relation can become Dying via either `juju destroy-relation` or
+    `juju destroy-service`. (Maybe; see services, above.)
+  * When a relation is destroyed with no units in scope, it will be removed
+    from state immediately rather than being set to Dying.
+  * When a relation becomes Dying, the UAs of units that have entered its scope
+    are responsible for cleanly departing the relation by running hooks and then
+    leaving relation scope.
+  * When the last unit leaves the scope of a Dying relation, it must remove the
+    relation from state.
+  * As noted above, the Dying relation may be the only thing keeping a Dying
+    service (different to that of the acting UA) from removal; so, relation
+    removal may also imply service removal.
+
+References
+----------
+
+OK, that was a bit of a hail of bullets, and the motivations for the above are
+perhaps not always clear. To consider it from another angle:
+
+  * Subordinate units reference principal units.
+  * Principal units reference machines.
+  * All units reference their services.
+  * All units reference the relations whose scopes they have joined.
+  * All relations reference the services they are part of.
+
+In every case above, where X references Y, the life state of an X may be
+sufficient to prevent a change in the life state of a Y; and, conversely, a
+life change in an X may be sufficient to cause a life change in a Y. (In only
+one case does the reverse hold -- that is, setting a service to Dying will
+cause its units' agents to individually set the units to Dying -- and this is
+just an implementation detail.)
+
+The following scrawl may help you to visualize the references in play:
+
+        +-----------+       +---------+
+    +-->| principal |------>| machine |
+    |   +-----------+       +---------+
+    |      |     |
+    |      |     +--------------+
+    |      |                    |
+    |      V                    V
+    |   +----------+       +---------+
+    |   | relation |------>| service |
+    |   +----------+       +---------+
+    |      A                    A
+    |      |                    |
+    |      |     +--------------+
+    |      |     |
+    |   +-------------+
+    +---| subordinate |
+        +-------------+
+
+...but is important to remember that it's only one view of the relationships
+involved, and that the user-centric view is quite different; from a user's
+perspective the influences appear to travel in the opposite direction:
+
+  * (destroying a machine "would" destroy its principals but that's disallowed)
+  * destroying a principal destroys all its subordinates
+  * (destroying a subordinate directly is impossible)
+  * destroying a service destroys all its units and relations
+  * destroying a container relation destroys all subordinates in the relation
+  * (destroying a global relation destroys nothing else)
+
+...and it takes a combination of these viewpoints to understand the detailed
+interactions laid out above.
+
+Agents
+------
+
+It may also be instructive to consider the responsibilities of the unit and
+machine agents. The unit agent is responsible for:
+
+  * detecting its service's Dying state, and setting its own Dying state.
+  * detecting its own Dying state, and:
+      * leaving relation scopes
+      * possibly removing those relations
+      * possibly removing some of those relations' services
+      * waiting for its subordinates' Dead states and
+          * removing the subordinates
+          * possibly removing their services
+      * finally setting its own Dead state
+
+The machine agent, meanwhile, is responsible for:
+
+  * detecting its principals' Dead states and:
+      * removing the principals
+      * possibly removing their services
+  * detecting its machine's Dying state, and setting it to Dead.
+
+Finally, the provisioning agent is responsible for:
+
+  * detecting machines' Dead states, and removing the machines.
+
+(Oh: it shoud be noted explicitly that the PA is responsible for provisioning
+Alive machines, that an MA is responsible for deploying Alive principals, that
+a principal UA is responsible for creating and deploying its Alive subordinates,
+and that all UAs are responsible for entering relevant relation scopes; but that
+side is easy, and the what-do-we-do-with-a-Dying-object problem STM to have the
+same tradeoffs regardless of object: the problem is still, and always, about
+how we shoud treat Dying units backed by persistent storage, and the answer
+remains easy... for now.)
+
+Implementation
+--------------
+
+We use the mgo/txn package for easy multi-document transactions against MongoDB.
+Without this, it would be very difficult to enforce the various conditions
+described above (consider, for example, the race between one client assigning a
+new principal to a machine that another client is trying to destroy). However,
+even with mgo/txn doing an awful lot of heavy lifting, actually landing a
+consistent transaction occasionally demands a somewhat taxing level of attention
+to detail.
+
+At the moment, the only type whose lifecycle has been implemented approximately
+correctly is relation... and even that's incomplete, because RelationUnit's
+LeaveScope method fails to take into account the possibility that it might be
+responsible for destroying a service as well as just a relation. But, I guess,
+we haven't started looking at service desctruction yet, so this is maybe not so
+bad.
+
+So -- IMO, none of this has yet been implemented to an adequate standard, and
+we're not short of detailed work to do. I'm currently focused on subordinates,
+which will demand correct implementations of:
+
+  * relation destruction (done)
+  * service destruction (not started)
+  * unit destruction (not started)
+
+...and if we're doing all those we really should do machine destruction too,
+especially considering it'll be easiest, but I'm not even going to think about
+that until i've dealt with the rest.
+
+Implementation issues
+---------------------
+
+As mentioned, we save ourselves some hassle by restricting machines to become
+Dying only when they have no units (rather than when none of their units are
+Alive). Fixing this will probably be trivial (we have to solve almost exactly
+the same problem for subordinate units wrt their principal), but since we don't
+*have* to worry about it yet I'm ignoring it.
+
+But there's a trickier one: as you will no doubt have observed, the unit is
+mired deep in dependencies, and a non-functioning unit agent can potentially
+paralyse whole chunks of the system [0]. So, we decided a while ago that we should
+support forcible unit destruction: that it should be possible to set a unit to
+Dead directly.
+
+However, the number of interactions between units and other objects is a little
+bit daunting. In the worst case, forcibly destroying a unit and keeping state
+consistent may require:
+
+  * that the unit leave several relation scopes
+  * that the unit remove several relations it had been in scope of
+  * that the unit remove some services on the other end of removed relations
+
+...*and*:
+
+  * that all the same things happen for all the unit's subordinates.
+
+To take a pessimistic view of the matter, "just" setting a unit to Dead may
+require hundreds of txn operations in a pathological case. However, none of
+the alternatives fill me with great joy; I am starting to feel that a big
+ugly bottleneck txn (specifically to deal with known-pathological cases) may
+in fact be the most reliable and comprehensible way to express the desired
+state change. More discussions on this topic will most assuredly come to pass.
+
+-----
+
+[0] Heh. Frankly, so can a paralysed MA. `juju destroy-machine --force`..? If unit
+force destruction seemed complex, the machine case is even worse because machines
+can (in theory, if not currently in practice) have several principal units. No, I
+haven't figured out how to deal with a paralysed PA either...

